=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-14 08:41:00 +0000
+++ bzrlib/builtins.py	2005-10-14 09:37:21 +0000
@@ -703,7 +703,8 @@
                      'show-ids', 'revision',
                      'long', 
                      Option('message',
-                            help='show revisions whose message matches this regexp'),
+                            help='show revisions whose message matches this regexp',
+                            type=str),
                      'short',]
     
     def run(self, filename=None, timezone='original',
@@ -716,7 +717,8 @@
             short=False):
         from bzrlib.log import log_formatter, show_log
         import codecs
-
+        assert isinstance(message, basestring), \
+            "invalid message argument %r" % message
         direction = (forward and 'forward') or 'reverse'
         
         if filename:
@@ -995,8 +997,12 @@
     # XXX: verbose currently does nothing
 
     takes_args = ['selected*']
-    takes_options = ['message', 'verbose', 'unchanged',
-                     Option('file', type=str, help='file containing commit message'),
+    takes_options = ['message', 'verbose', 
+                     Option('unchanged',
+                            help='commit even if nothing has changed'),
+                     Option('file', type=str, 
+                            argname='msgfile',
+                            help='file containing commit message'),
                      ]
     aliases = ['ci', 'checkin']
 

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-10-14 08:36:56 +0000
+++ bzrlib/commands.py	2005-10-14 09:37:21 +0000
@@ -182,6 +182,7 @@
 
         Maps from long option name to option object."""
         r = dict()
+        r['help'] = Option.OPTIONS['help']
         for o in self.takes_options:
             if not isinstance(o, Option):
                 o = Option.OPTIONS[o]
@@ -190,19 +191,17 @@
 
     def run_argv(self, argv):
         """Parse command line and run."""
-        args, opts = parse_args(argv)
-
+        args, opts = parse_args(self, argv)
         if 'help' in opts:  # e.g. bzr add --help
             from bzrlib.help import help_on_command
             help_on_command(self.name())
             return 0
-
+        # XXX: This should be handled by the parser
         allowed_names = self.options().keys()
         for oname in opts:
             if oname not in allowed_names:
                 raise BzrCommandError("option '--%s' is not allowed for command %r"
                                       % (oname, self.name()))
-
         # mix arguments and options into one dictionary
         cmdargs = _match_argform(self.name(), self.takes_args, args)
         cmdopts = {}
@@ -269,37 +268,19 @@
         parsed = [spec, None]
     return parsed
 
-def parse_args(argv):
+def parse_args(command, argv):
     """Parse command line.
     
     Arguments and options are parsed at this level before being passed
     down to specific command handlers.  This routine knows, from a
     lookup table, something about the available options, what optargs
     they take, and which commands will accept them.
-
-    >>> parse_args('--help'.split())
-    ([], {'help': True})
-    >>> parse_args('help -- --invalidcmd'.split())
-    (['help', '--invalidcmd'], {})
-    >>> parse_args('--version'.split())
-    ([], {'version': True})
-    >>> parse_args('status --all'.split())
-    (['status'], {'all': True})
-    >>> parse_args('commit --message=biter'.split())
-    (['commit'], {'message': u'biter'})
-    >>> parse_args('log -r 500'.split())
-    (['log'], {'revision': [<RevisionSpec_int 500>]})
-    >>> parse_args('log -r500..600'.split())
-    (['log'], {'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
-    >>> parse_args('log -vr500..600'.split())
-    (['log'], {'verbose': True, 'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
-    >>> parse_args('log -rrevno:500..600'.split()) #the r takes an argument
-    (['log'], {'revision': [<RevisionSpec_revno revno:500>, <RevisionSpec_int 600>]})
     """
     # TODO: chop up this beast
     args = []
     opts = {}
 
+    cmd_options = command.options()
     argsover = False
     while argv:
         a = argv.pop(0)
@@ -317,8 +298,9 @@
                     optname, optarg = a[2:].split('=', 1)
                 else:
                     optname = a[2:]
-                if optname not in Option.OPTIONS:
-                    raise BzrError('unknown long option %r' % a)
+                if optname not in cmd_options:
+                    raise BzrCommandError('unknown long option %r for command %s' 
+                            % (a, command.name))
             else:
                 shortopt = a[1:]
                 if shortopt in Option.SHORT_OPTIONS:
@@ -355,7 +337,8 @@
                 # XXX: Do we ever want to support this, e.g. for -r?
                 raise BzrError('repeated option %r' % a)
                 
-            optargfn = Option.OPTIONS[optname].type
+            option_obj = cmd_options[optname]
+            optargfn = option_obj.type
             if optargfn:
                 if optarg == None:
                     if not argv:
@@ -369,12 +352,9 @@
                 opts[optname] = True
         else:
             args.append(a)
-
     return args, opts
 
 
-
-
 def _match_argform(cmd, takes_args, args):
     argdict = {}
 

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2005-10-14 08:37:15 +0000
+++ bzrlib/help.py	2005-10-14 09:37:21 +0000
@@ -125,6 +125,8 @@
     outfile.write('\noptions:\n')
     for option_name, option in sorted(options.items()):
         l = '    --' + option_name
+        if option.type is not None:
+            l += ' ' + option.argname.upper()
         short_name = option.short_name()
         if short_name:
             assert len(short_name) == 1

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2005-10-14 08:45:55 +0000
+++ bzrlib/option.py	2005-10-14 09:37:21 +0000
@@ -112,7 +112,7 @@
     OPTIONS = {}
     SHORT_OPTIONS = {}
 
-    def __init__(self, name, help='', type=None):
+    def __init__(self, name, help='', type=None, argname=None):
         """Make a new command option.
 
         name -- regular name of the command, used in the double-dash
@@ -123,12 +123,19 @@
 
         type -- function called to parse the option argument, or 
             None (default) if this option doesn't take an argument.
+
+        argname -- name of option argument, if any
         """
         # TODO: perhaps a subclass that automatically does 
         # --option, --no-option for reversable booleans
         self.name = name
         self.help = help
         self.type = type
+        if type is None:
+            assert argname is None
+        elif argname is None:
+            argname = 'ARG'
+        self.argname = argname
 
     def short_name(self):
         """Return the single character option for this command, if any.
@@ -157,12 +164,13 @@
 _global_option('short')
 _global_option('show-ids', 
                help='show internal object ids')
-_global_option('timezone', type=str)
-_global_option('verbose',)
-##               help='display more information')
+_global_option('timezone', 
+               type=str,
+               help='display timezone as local, original, or utc')
+_global_option('verbose',
+               help='display more information')
 _global_option('version')
 _global_option('email')
-_global_option('unchanged')
 _global_option('update')
 _global_option('long')
 _global_option('root', type=str)

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-10-14 05:53:28 +0000
+++ bzrlib/selftest/__init__.py	2005-10-14 09:37:21 +0000
@@ -520,6 +520,7 @@
                    'bzrlib.selftest.testtestament',
                    'bzrlib.selftest.testannotate',
                    'bzrlib.selftest.testrevprops',
+                   'bzrlib.selftest.testoptions',
                    ]
 
     for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-10-14 08:45:55 +0000
+++ bzrlib/selftest/blackbox.py	2005-10-14 09:37:21 +0000
@@ -218,16 +218,6 @@
         self.log(tmp_output)
         self.assertEquals(output, tmp_output)
 
-    def test_option_help(self):
-        """Options have help strings."""
-        out, err = self.run_bzr_captured(['commit', '--help'])
-        self.assertContainsRe(out, r'--file.*file containing commit message')
-
-    def test_option_help_global(self):
-        """Global options have help strings."""
-        out, err = self.run_bzr_captured(['help', 'status'])
-        self.assertContainsRe(out, r'--show-ids.*show internal object')
-
     def example_branch(test):
         test.runbzr('init')
         file('hello', 'wt').write('foo')
@@ -810,3 +800,5 @@
         self.run_bzr('branch', url, 'to')
         branch = Branch.open('to')
         self.assertEqual(1, len(branch.revision_history()))
+
+

=== added file 'bzrlib/selftest/testoptions.py'
--- bzrlib/selftest/testoptions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testoptions.py	2005-10-14 09:37:21 +0000
@@ -0,0 +1,47 @@
+# (C) 2005 Canonical
+
+from bzrlib.selftest import TestCase
+from bzrlib.commands import Command, parse_args
+from bzrlib.builtins import cmd_commit, cmd_log, cmd_status
+
+
+class OptionTests(TestCase):
+    """Command-line option tests"""
+
+    def test_parse_args(self):
+        """Option parser"""
+        eq = self.assertEquals
+        eq(parse_args(cmd_commit(), ['--help']),
+           ([], {'help': True}))
+        eq(parse_args(cmd_status(), ['--all']),
+           ([], {'all': True}))
+        eq(parse_args(cmd_commit(), ['--message=biter']),
+           ([], {'message': 'biter'}))
+        ## eq(parse_args(cmd_log(),  '-r 500'.split()),
+        ##   ([], {'revision': RevisionSpec_int(500)}))
+
+    def test_option_help(self):
+        """Options have help strings."""
+        out, err = self.run_bzr_captured(['commit', '--help'])
+        self.assertContainsRe(out, r'--file.*file containing commit message')
+
+    def test_option_help_global(self):
+        """Global options have help strings."""
+        out, err = self.run_bzr_captured(['help', 'status'])
+        self.assertContainsRe(out, r'--show-ids.*show internal object')
+
+    def test_option_arg_help(self):
+        """Help message shows option arguments."""
+        out, err = self.run_bzr_captured(['help', 'commit'])
+        self.assertEquals(err, '')
+        self.assertContainsRe(out, r'--file[ =]MSGFILE')
+
+
+#     >>> parse_args('log -r 500'.split())
+#     (['log'], {'revision': [<RevisionSpec_int 500>]})
+#     >>> parse_args('log -r500..600'.split())
+#     (['log'], {'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+#     >>> parse_args('log -vr500..600'.split())
+#     (['log'], {'verbose': True, 'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+#     >>> parse_args('log -rrevno:500..600'.split()) #the r takes an argument
+#     (['log'], {'revision': [<RevisionSpec_revno revno:500>, <RevisionSpec_int 600>]})

